#line 2 "predicate_lexer.c"

#line 4 "predicate_lexer.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t;
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else   /* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif  /* defined (__STDC__) */
#endif  /* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void *yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE Solidrestart(yyin ,yyscanner )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

#define YY_LESS_LINENO(n)

/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
    do \
    { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        *yy_cp = yyg->yy_hold_char; \
        YY_RESTORE_YY_MORE_OFFSET \
        yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
        YY_DO_BEFORE_ACTION; /* set up yytext again */ \
    } \
    while ( 0 )

#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state {
    FILE *yy_input_file;

    char *yy_ch_buf;        /* input buffer */
    char *yy_buf_pos;       /* current position in input buffer */

    /* Size of input buffer in bytes, not including room for EOB
     * characters.
     */
    yy_size_t yy_buf_size;

    /* Number of characters read into yy_ch_buf, not including EOB
     * characters.
     */
    int yy_n_chars;

    /* Whether we "own" the buffer - i.e., we know we created it,
     * and can realloc() it to grow it, and should free() it to
     * delete it.
     */
    int yy_is_our_buffer;

    /* Whether this is an "interactive" input source; if so, and
     * if we're using stdio for input, then we want to use getc()
     * instead of fread(), to make sure we stop fetching input after
     * each newline.
     */
    int yy_is_interactive;

    /* Whether we're considered to be at the beginning of a line.
     * If so, '^' rules will be active on the next match, otherwise
     * not.
     */
    int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

    /* Whether to try to fill the input buffer when we reach the
     * end of it.
     */
    int yy_fill_buffer;

    int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
    /* When an EOF's been seen but there's still some text to process
     * then we mark the buffer as YY_EOF_PENDING, to indicate that we
     * shouldn't try reading from the input source any more.  We might
     * still have a bunch of tokens to match, though, because of
     * possible backing-up.
     *
     * When we actually see the EOF, we change the status to "new"
     * (via Solidrestart()), so that the user can continue scanning by
     * just pointing yyin at a new input file.
     */
#define YY_BUFFER_EOF_PENDING 2

};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                            ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                            : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

void Solidrestart(FILE *input_file, yyscan_t yyscanner);
void Solid_switch_to_buffer(YY_BUFFER_STATE new_buffer, yyscan_t yyscanner);
YY_BUFFER_STATE Solid_create_buffer(FILE *file, int size, yyscan_t yyscanner);
void Solid_delete_buffer(YY_BUFFER_STATE b, yyscan_t yyscanner);
void Solid_flush_buffer(YY_BUFFER_STATE b, yyscan_t yyscanner);
void Solidpush_buffer_state(YY_BUFFER_STATE new_buffer, yyscan_t yyscanner);
void Solidpop_buffer_state(yyscan_t yyscanner);

static void Solidensure_buffer_stack(yyscan_t yyscanner);
static void Solid_load_buffer_state(yyscan_t yyscanner);
static void Solid_init_buffer(YY_BUFFER_STATE b, FILE *file, yyscan_t yyscanner);

#define YY_FLUSH_BUFFER Solid_flush_buffer(YY_CURRENT_BUFFER ,yyscanner)

YY_BUFFER_STATE Solid_scan_buffer(char *base, yy_size_t size, yyscan_t yyscanner);
YY_BUFFER_STATE Solid_scan_string(yyconst char *yy_str, yyscan_t yyscanner);
YY_BUFFER_STATE Solid_scan_bytes(yyconst char *bytes, int len, yyscan_t yyscanner);

void *Solidalloc(yy_size_t, yyscan_t yyscanner);
void *Solidrealloc(void *, yy_size_t, yyscan_t yyscanner);
void Solidfree(void *, yyscan_t yyscanner);

#define yy_new_buffer Solid_create_buffer

#define yy_set_interactive(is_interactive) \
    { \
        if ( ! YY_CURRENT_BUFFER ){ \
            Solidensure_buffer_stack (yyscanner); \
            YY_CURRENT_BUFFER_LVALUE =    \
                                          Solid_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
        } \
        YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
    }

#define yy_set_bol(at_bol) \
    { \
        if ( ! YY_CURRENT_BUFFER ){\
            Solidensure_buffer_stack (yyscanner); \
            YY_CURRENT_BUFFER_LVALUE =    \
                                          Solid_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
        } \
        YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
    }

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

static yy_state_type yy_get_previous_state(yyscan_t yyscanner);
static yy_state_type yy_try_NUL_trans(yy_state_type current_state, yyscan_t yyscanner);
static int yy_get_next_buffer(yyscan_t yyscanner);
static void yy_fatal_error(yyconst char msg[], yyscan_t yyscanner);

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
    yyg->yytext_ptr = yy_bp; \
    yyleng = (size_t) (yy_cp - yy_bp); \
    yyg->yy_hold_char = *yy_cp; \
    *yy_cp = '\0'; \
    yyg->yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 16
#define YY_END_OF_BUFFER 17
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info {
    flex_int32_t yy_verify;
    flex_int32_t yy_nxt;
};
static yyconst flex_int16_t yy_accept[41] = {
    0,
    0,    0,   17,   15,   14,   14,    2,   15,   13,   15,
    13,   10,   15,   12,   12,   12,   12,   12,   12,   14,
    0,    8,    9,   11,    0,   10,    1,   12,   12,   12,
    5,    4,   12,    3,   12,   12,   12,    6,    7,    0
};

static yyconst flex_int32_t yy_ec[256] = {
    0,
    1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    2,    1,    1,    1,    1,    1,    4,    5,    1,
    1,    1,    1,    6,    7,    8,    1,    9,    9,    9,
    9,    9,    9,    9,    9,    9,    9,    1,    1,    1,
    10,    1,    1,    1,   11,   12,   12,   13,   14,   15,
    12,   12,   16,   12,   12,   17,   12,   18,   19,   12,
    12,   20,   21,   22,   23,   12,   12,   12,   12,   12,
    24,    1,   25,    1,    1,    1,   11,   12,   12,   13,

    14,   15,   12,   12,   16,   12,   12,   17,   12,   18,
    19,   12,   12,   20,   21,   22,   23,   12,   12,   12,
    12,   12,   26,    1,   27,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1
};

static yyconst flex_int32_t yy_meta[28] = {
    0,
    1,    1,    1,    1,    1,    1,    2,    1,    2,    1,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    1,    1,    1,    1
};

static yyconst flex_int16_t yy_base[43] = {
    0,
    0,    0,   60,   61,   26,   28,   61,   54,   61,   49,
    48,   24,   46,   37,    0,   43,   32,   32,   31,   32,
    45,   61,   40,   39,   38,   28,   61,    0,   33,   28,
    0,    0,   21,    0,   22,   28,   27,    0,    0,   61,
    37,   38
};

static yyconst flex_int16_t yy_def[43] = {
    0,
    40,    1,   40,   40,   40,   40,   40,   41,   40,   40,
    40,   40,   40,   42,   42,   42,   42,   42,   42,   40,
    41,   40,   40,   40,   40,   40,   40,   42,   42,   42,
    42,   42,   42,   42,   42,   42,   42,   42,   42,    0,
    40,   40
};

static yyconst flex_int16_t yy_nxt[89] = {
    0,
    4,    5,    6,    7,    8,    9,   10,   11,   12,   13,
    14,   15,   15,   15,   16,   17,   15,   15,   18,   15,
    15,   19,   15,    9,    9,    9,    9,   20,   20,   20,
    20,   25,   26,   20,   20,   25,   26,   21,   21,   28,
    39,   38,   37,   36,   35,   34,   24,   24,   23,   22,
    33,   32,   31,   30,   29,   27,   24,   23,   22,   40,
    3,   40,   40,   40,   40,   40,   40,   40,   40,   40,
    40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
    40,   40,   40,   40,   40,   40,   40,   40
};

static yyconst flex_int16_t yy_chk[89] = {
    0,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    5,    5,    6,
    6,   12,   12,   20,   20,   26,   26,   41,   41,   42,
    37,   36,   35,   33,   30,   29,   25,   24,   23,   21,
    19,   18,   17,   16,   14,   13,   11,   10,    8,    3,
    40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
    40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
    40,   40,   40,   40,   40,   40,   40,   40
};

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "predicate_lexer.l"
#line 2 "predicate_lexer.l"
#include "predicate_parser.h"
#include "predicateparse.h"
#include <string.h>
#include <stdlib.h>
#define YY_NO_UNPUT

int Solidwrap(yyscan_t _scanner);
void PredicateParse_initLexer(const char *_code, yyscan_t _scanner);
char *PredicateParse_putSymbol(char *_name);
char *PredicateParse_putString(char *_str);
void PredicateParse_initFlex(const char *_code, yyscan_t _scanner);

#line 480 "predicate_lexer.c"

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE *yy_buffer_stack;  /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char *yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE *yylval_r;

}; /* end struct yyguts_t */

static int yy_init_globals(yyscan_t yyscanner);

/* This must go here because YYSTYPE and YYLTYPE are included
 * from bison output in section 1.*/
#    define yylval yyg->yylval_r

int Solidlex_init(yyscan_t *scanner);

int Solidlex_init_extra(YY_EXTRA_TYPE user_defined, yyscan_t *scanner);

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int Solidlex_destroy(yyscan_t yyscanner);

int Solidget_debug(yyscan_t yyscanner);

void Solidset_debug(int debug_flag, yyscan_t yyscanner);

YY_EXTRA_TYPE Solidget_extra(yyscan_t yyscanner);

void Solidset_extra(YY_EXTRA_TYPE user_defined, yyscan_t yyscanner);

FILE *Solidget_in(yyscan_t yyscanner);

void Solidset_in(FILE *in_str, yyscan_t yyscanner);

FILE *Solidget_out(yyscan_t yyscanner);

void Solidset_out(FILE *out_str, yyscan_t yyscanner);

int Solidget_leng(yyscan_t yyscanner);

char *Solidget_text(yyscan_t yyscanner);

int Solidget_lineno(yyscan_t yyscanner);

void Solidset_lineno(int line_number, yyscan_t yyscanner);

YYSTYPE *Solidget_lval(yyscan_t yyscanner);

void Solidset_lval(YYSTYPE *yylval_param, yyscan_t yyscanner);

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int Solidwrap(yyscan_t yyscanner);
#else
extern int Solidwrap(yyscan_t yyscanner);
#endif
#endif

static void yyunput(int c, char *buf_ptr, yyscan_t yyscanner);

#ifndef yytext_ptr
static void yy_flex_strncpy(char *, yyconst char *, int, yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen(yyconst char *, yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput(yyscan_t yyscanner);
#else
static int input(yyscan_t yyscanner);
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
    if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
    { \
        int c = '*'; \
        size_t n; \
        for ( n = 0; n < max_size && \
                (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
            buf[n] = (char) c; \
        if ( c == '\n' ) \
            buf[n++] = (char) c; \
        if ( c == EOF && ferror( yyin ) ) \
            YY_FATAL_ERROR( "input in flex scanner failed" ); \
        result = n; \
    } \
    else \
    { \
        errno=0; \
        while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
        { \
            if( errno != EINTR) \
            { \
                YY_FATAL_ERROR( "input in flex scanner failed" ); \
                break; \
            } \
            errno=0; \
            clearerr(yyin); \
        } \
    }\
    \
     
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int Solidlex \
(YYSTYPE *yylval_param, yyscan_t yyscanner);

#define YY_DECL int Solidlex \
    (YYSTYPE * yylval_param , yyscan_t yyscanner)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
    YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL {
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

#line 25 "predicate_lexer.l"

#line 721 "predicate_lexer.c"

    yylval = yylval_param;

    if (!yyg->yy_init)
    {
        yyg->yy_init = 1;

#ifdef YY_USER_INIT
        YY_USER_INIT;
#endif

        if (! yyg->yy_start) {
            yyg->yy_start = 1;    /* first start state */
        }

        if (! yyin) {
            yyin = stdin;
        }

        if (! yyout) {
            yyout = stdout;
        }

        if (! YY_CURRENT_BUFFER) {
            Solidensure_buffer_stack(yyscanner);
            YY_CURRENT_BUFFER_LVALUE =
                Solid_create_buffer(yyin, YY_BUF_SIZE, yyscanner);
        }

        Solid_load_buffer_state(yyscanner);
    }

    while (1)        /* loops until end-of-file is reached */
    {
        yy_cp = yyg->yy_c_buf_p;

        /* Support of yytext. */
        *yy_cp = yyg->yy_hold_char;

        /* yy_bp points to the position in yy_ch_buf of the start of
         * the current run.
         */
        yy_bp = yy_cp;

        yy_current_state = yyg->yy_start;
    yy_match:
        do {
            register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
            if (yy_accept[yy_current_state]) {
                yyg->yy_last_accepting_state = yy_current_state;
                yyg->yy_last_accepting_cpos = yy_cp;
            }
            while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
                yy_current_state = (int) yy_def[yy_current_state];
                if (yy_current_state >= 41) {
                    yy_c = yy_meta[(unsigned int) yy_c];
                }
            }
            yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
            ++yy_cp;
        } while (yy_current_state != 40);
        yy_cp = yyg->yy_last_accepting_cpos;
        yy_current_state = yyg->yy_last_accepting_state;

    yy_find_action:
        yy_act = yy_accept[yy_current_state];

        YY_DO_BEFORE_ACTION;

    do_action:  /* This label is used only to access EOF actions. */

        switch (yy_act) {
        /* beginning of action switch */
        case 0: /* must back up */
            /* undo the effects of YY_DO_BEFORE_ACTION */
            *yy_cp = yyg->yy_hold_char;
            yy_cp = yyg->yy_last_accepting_cpos;
            yy_current_state = yyg->yy_last_accepting_state;
            goto yy_find_action;

        case 1:
            YY_RULE_SETUP
#line 27 "predicate_lexer.l" 
            { return EQ; }
            YY_BREAK
        case 2:
            YY_RULE_SETUP
#line 28 "predicate_lexer.l" 
            { return MASK; }
            YY_BREAK
        case 3:
            YY_RULE_SETUP
#line 30 "predicate_lexer.l" 
            { return AND; }
            YY_BREAK
        case 4:
            YY_RULE_SETUP
#line 31 "predicate_lexer.l" 
            { return OR; }
            YY_BREAK
        case 5:
            YY_RULE_SETUP
#line 32 "predicate_lexer.l" 
            { return IS; }
            YY_BREAK
        case 6:
            YY_RULE_SETUP
#line 34 "predicate_lexer.l" 
            { yylval->valb = 1; return VAL_BOOL; }
            YY_BREAK
        case 7:
            YY_RULE_SETUP
#line 35 "predicate_lexer.l" 
            { yylval->valb = 0; return VAL_BOOL; }
            YY_BREAK
        case 8:
            /* rule 8 can match eol */
            YY_RULE_SETUP
#line 37 "predicate_lexer.l" 
            { yylval->name = PredicateParse_putString(yytext); return VAL_STRING; }
            YY_BREAK
        case 9:
            YY_RULE_SETUP
#line 39 "predicate_lexer.l" 
            { yylval->vali = atoi(yytext); return VAL_NUM; }
            YY_BREAK
        case 10:
            YY_RULE_SETUP
#line 40 "predicate_lexer.l" 
            { yylval->vali = atoi(yytext); return VAL_NUM; }
            YY_BREAK
        case 11:
            YY_RULE_SETUP
#line 42 "predicate_lexer.l" 
            { yylval->vald = atof(yytext); return VAL_FLOAT; }
            YY_BREAK
        case 12:
            YY_RULE_SETUP
#line 44 "predicate_lexer.l" 
            { yylval->name = PredicateParse_putSymbol(yytext); return VAL_ID; }
            YY_BREAK
        case 13:
            YY_RULE_SETUP
#line 46 "predicate_lexer.l" 
            { yylval->name = 0; return (int)(*yytext); }
            YY_BREAK
        case 14:
            /* rule 14 can match eol */
            YY_RULE_SETUP
#line 48 "predicate_lexer.l"
            /* eat up whitespace */
            YY_BREAK
        case 15:
            YY_RULE_SETUP
#line 50 "predicate_lexer.l" 
            { PredicateLexer_unknownToken(yytext); }
            YY_BREAK
        case 16:
            YY_RULE_SETUP
#line 52 "predicate_lexer.l"
            ECHO;
            YY_BREAK
#line 884 "predicate_lexer.c"
        case YY_STATE_EOF(INITIAL):
            yyterminate();

        case YY_END_OF_BUFFER: {
            /* Amount of text matched not including the EOB char. */
            int yy_amount_of_matched_text = (int)(yy_cp - yyg->yytext_ptr) - 1;

            /* Undo the effects of YY_DO_BEFORE_ACTION. */
            *yy_cp = yyg->yy_hold_char;
            YY_RESTORE_YY_MORE_OFFSET

            if (YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW) {
                /* We're scanning a new file or input source.  It's
                 * possible that this happened because the user
                 * just pointed yyin at a new source and called
                 * Solidlex().  If so, then we have to assure
                 * consistency between YY_CURRENT_BUFFER and our
                 * globals.  Here is the right place to do so, because
                 * this is the first action (other than possibly a
                 * back-up) that will match for the new input source.
                 */
                yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
                YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
                YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
            }

            /* Note that here we test for yy_c_buf_p "<=" to the position
             * of the first EOB in the buffer, since yy_c_buf_p will
             * already have been incremented past the NUL character
             * (since all states make transitions on EOB to the
             * end-of-buffer state).  Contrast this with the test
             * in input().
             */
            if (yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars]) {
                /* This was really a NUL. */
                yy_state_type yy_next_state;

                yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

                yy_current_state = yy_get_previous_state(yyscanner);

                /* Okay, we're now positioned to make the NUL
                 * transition.  We couldn't have
                 * yy_get_previous_state() go ahead and do it
                 * for us because it doesn't know how to deal
                 * with the possibility of jamming (and we don't
                 * want to build jamming into it because then it
                 * will run more slowly).
                 */

                yy_next_state = yy_try_NUL_trans(yy_current_state, yyscanner);

                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

                if (yy_next_state) {
                    /* Consume the NUL. */
                    yy_cp = ++yyg->yy_c_buf_p;
                    yy_current_state = yy_next_state;
                    goto yy_match;
                }

                else {
                    yy_cp = yyg->yy_last_accepting_cpos;
                    yy_current_state = yyg->yy_last_accepting_state;
                    goto yy_find_action;
                }
            }

            else switch (yy_get_next_buffer(yyscanner)) {
                case EOB_ACT_END_OF_FILE: {
                    yyg->yy_did_buffer_switch_on_eof = 0;

                    if (Solidwrap(yyscanner)) {
                        /* Note: because we've taken care in
                         * yy_get_next_buffer() to have set up
                         * yytext, we can now set up
                         * yy_c_buf_p so that if some total
                         * hoser (like flex itself) wants to
                         * call the scanner after we return the
                         * YY_NULL, it'll still work - another
                         * YY_NULL will get returned.
                         */
                        yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

                        yy_act = YY_STATE_EOF(YY_START);
                        goto do_action;
                    }

                    else {
                        if (! yyg->yy_did_buffer_switch_on_eof) {
                            YY_NEW_FILE;
                        }
                    }
                    break;
                }

                case EOB_ACT_CONTINUE_SCAN:
                    yyg->yy_c_buf_p =
                        yyg->yytext_ptr + yy_amount_of_matched_text;

                    yy_current_state = yy_get_previous_state(yyscanner);

                    yy_cp = yyg->yy_c_buf_p;
                    yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                    goto yy_match;

                case EOB_ACT_LAST_MATCH:
                    yyg->yy_c_buf_p =
                        &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

                    yy_current_state = yy_get_previous_state(yyscanner);

                    yy_cp = yyg->yy_c_buf_p;
                    yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                    goto yy_find_action;
                }
            break;
        }

        default:
            YY_FATAL_ERROR(
                "fatal flex scanner internal error--no action found");
        } /* end of action switch */
    } /* end of scanning one token */
} /* end of Solidlex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *  EOB_ACT_LAST_MATCH -
 *  EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *  EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
    register char *source = yyg->yytext_ptr;
    register int number_to_move, i;
    int ret_val;

    if (yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1])
        YY_FATAL_ERROR(
            "fatal flex scanner internal error--end of buffer missed");

    if (YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0) {
        /* Don't try to fill the buffer, so this is an EOF. */
        if (yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1) {
            /* We matched a single character, the EOB, so
             * treat this as a final EOF.
             */
            return EOB_ACT_END_OF_FILE;
        }

        else {
            /* We matched some text prior to the EOB, first
             * process it.
             */
            return EOB_ACT_LAST_MATCH;
        }
    }

    /* Try to read more data. */

    /* First move last chars to start of buffer. */
    number_to_move = (int)(yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

    for (i = 0; i < number_to_move; ++i) {
        *(dest++) = *(source++);
    }

    if (YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING)
        /* don't do the read, it's not guaranteed to return an EOF,
         * just force an EOF
         */
    {
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;
    }

    else {
        int num_to_read =
            YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

        while (num_to_read <= 0) {
            /* Not enough room in the buffer - grow it. */

            /* just a shorter name for the current buffer */
            YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

            int yy_c_buf_p_offset =
                (int)(yyg->yy_c_buf_p - b->yy_ch_buf);

            if (b->yy_is_our_buffer) {
                int new_size = b->yy_buf_size * 2;

                if (new_size <= 0) {
                    b->yy_buf_size += b->yy_buf_size / 8;
                } else {
                    b->yy_buf_size *= 2;
                }

                b->yy_ch_buf = (char *)
                               /* Include room in for 2 EOB chars. */
                               Solidrealloc((void *) b->yy_ch_buf, b->yy_buf_size + 2, yyscanner);
            } else
                /* Can't grow it, we don't own it. */
            {
                b->yy_ch_buf = 0;
            }

            if (! b->yy_ch_buf)
                YY_FATAL_ERROR(
                    "fatal error - scanner input buffer overflow");

            yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

            num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
                          number_to_move - 1;

        }

        if (num_to_read > YY_READ_BUF_SIZE) {
            num_to_read = YY_READ_BUF_SIZE;
        }

        /* Read in more data. */
        YY_INPUT((&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
                 yyg->yy_n_chars, (size_t) num_to_read);

        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
    }

    if (yyg->yy_n_chars == 0) {
        if (number_to_move == YY_MORE_ADJ) {
            ret_val = EOB_ACT_END_OF_FILE;
            Solidrestart(yyin, yyscanner);
        }

        else {
            ret_val = EOB_ACT_LAST_MATCH;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
                YY_BUFFER_EOF_PENDING;
        }
    }

    else {
        ret_val = EOB_ACT_CONTINUE_SCAN;
    }

    if ((yy_size_t)(yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
        /* Extend the array by 50%, plus the number we really need. */
        yy_size_t new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
        YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) Solidrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, new_size, yyscanner);
        if (! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf) {
            YY_FATAL_ERROR("out of dynamic memory in yy_get_next_buffer()");
        }
    }

    yyg->yy_n_chars += number_to_move;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

    yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

    return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state(yyscan_t yyscanner)
{
    register yy_state_type yy_current_state;
    register char *yy_cp;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    yy_current_state = yyg->yy_start;

    for (yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp) {
        register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
        if (yy_accept[yy_current_state]) {
            yyg->yy_last_accepting_state = yy_current_state;
            yyg->yy_last_accepting_cpos = yy_cp;
        }
        while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
            yy_current_state = (int) yy_def[yy_current_state];
            if (yy_current_state >= 41) {
                yy_c = yy_meta[(unsigned int) yy_c];
            }
        }
        yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    }

    return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *  next_state = yy_try_NUL_trans( current_state );
 */
static yy_state_type yy_try_NUL_trans(yy_state_type yy_current_state, yyscan_t yyscanner)
{
    register int yy_is_jam;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner; /* This var may be unused depending upon options. */
    register char *yy_cp = yyg->yy_c_buf_p;

    register YY_CHAR yy_c = 1;
    if (yy_accept[yy_current_state]) {
        yyg->yy_last_accepting_state = yy_current_state;
        yyg->yy_last_accepting_cpos = yy_cp;
    }
    while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
        yy_current_state = (int) yy_def[yy_current_state];
        if (yy_current_state >= 41) {
            yy_c = yy_meta[(unsigned int) yy_c];
        }
    }
    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    yy_is_jam = (yy_current_state == 40);

    return yy_is_jam ? 0 : yy_current_state;
}

static void yyunput(int c, register char *yy_bp, yyscan_t yyscanner)
{
    register char *yy_cp;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    yy_cp = yyg->yy_c_buf_p;

    /* undo effects of setting up yytext */
    *yy_cp = yyg->yy_hold_char;

    if (yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2) {
        /* need to shift things up to make room */
        /* +2 for EOB chars. */
        register int number_to_move = yyg->yy_n_chars + 2;
        register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
                                  YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
        register char *source =
            &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

        while (source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf) {
            *--dest = *--source;
        }

        yy_cp += (int)(dest - source);
        yy_bp += (int)(dest - source);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
            yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

        if (yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2) {
            YY_FATAL_ERROR("flex scanner push-back overflow");
        }
    }

    *--yy_cp = (char) c;

    yyg->yytext_ptr = yy_bp;
    yyg->yy_hold_char = *yy_cp;
    yyg->yy_c_buf_p = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput(yyscan_t yyscanner)
#else
static int input(yyscan_t yyscanner)
#endif

{
    int c;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    *yyg->yy_c_buf_p = yyg->yy_hold_char;

    if (*yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR) {
        /* yy_c_buf_p now points to the character we want to return.
         * If this occurs *before* the EOB characters, then it's a
         * valid NUL; if not, then we've hit the end of the buffer.
         */
        if (yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars])
            /* This was really a NUL. */
        {
            *yyg->yy_c_buf_p = '\0';
        }

        else {
            /* need more input */
            int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
            ++yyg->yy_c_buf_p;

            switch (yy_get_next_buffer(yyscanner)) {
            case EOB_ACT_LAST_MATCH:
                /* This happens because yy_g_n_b()
                 * sees that we've accumulated a
                 * token and flags that we need to
                 * try matching the token before
                 * proceeding.  But for input(),
                 * there's no matching to consider.
                 * So convert the EOB_ACT_LAST_MATCH
                 * to EOB_ACT_END_OF_FILE.
                 */

                /* Reset buffer status. */
                Solidrestart(yyin, yyscanner);

            /*FALLTHROUGH*/

            case EOB_ACT_END_OF_FILE: {
                if (Solidwrap(yyscanner)) {
                    return EOF;
                }

                if (! yyg->yy_did_buffer_switch_on_eof) {
                    YY_NEW_FILE;
                }
#ifdef __cplusplus
                return yyinput(yyscanner);
#else
                return input(yyscanner);
#endif
            }

            case EOB_ACT_CONTINUE_SCAN:
                yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
                break;
            }
        }
    }

    c = *(unsigned char *) yyg->yy_c_buf_p; /* cast for 8-bit char's */
    *yyg->yy_c_buf_p = '\0';    /* preserve yytext */
    yyg->yy_hold_char = *++yyg->yy_c_buf_p;

    return c;
}
#endif  /* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
void Solidrestart(FILE *input_file, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    if (! YY_CURRENT_BUFFER) {
        Solidensure_buffer_stack(yyscanner);
        YY_CURRENT_BUFFER_LVALUE =
            Solid_create_buffer(yyin, YY_BUF_SIZE, yyscanner);
    }

    Solid_init_buffer(YY_CURRENT_BUFFER, input_file, yyscanner);
    Solid_load_buffer_state(yyscanner);
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
void Solid_switch_to_buffer(YY_BUFFER_STATE  new_buffer, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    /* TODO. We should be able to replace this entire function body
     * with
     *      Solidpop_buffer_state();
     *      Solidpush_buffer_state(new_buffer);
     */
    Solidensure_buffer_stack(yyscanner);
    if (YY_CURRENT_BUFFER == new_buffer) {
        return;
    }

    if (YY_CURRENT_BUFFER) {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
    }

    YY_CURRENT_BUFFER_LVALUE = new_buffer;
    Solid_load_buffer_state(yyscanner);

    /* We don't actually know whether we did this switch during
     * EOF (Solidwrap()) processing, but the only time this flag
     * is looked at is after Solidwrap() is called, so it's safe
     * to go ahead and always set it.
     */
    yyg->yy_did_buffer_switch_on_eof = 1;
}

static void Solid_load_buffer_state(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
    yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
    yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
    yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
YY_BUFFER_STATE Solid_create_buffer(FILE *file, int  size, yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;

    b = (YY_BUFFER_STATE) Solidalloc(sizeof(struct yy_buffer_state), yyscanner);
    if (! b) {
        YY_FATAL_ERROR("out of dynamic memory in Solid_create_buffer()");
    }

    b->yy_buf_size = size;

    /* yy_ch_buf has to be 2 characters longer than the size given because
     * we need to put in 2 end-of-buffer characters.
     */
    b->yy_ch_buf = (char *) Solidalloc(b->yy_buf_size + 2, yyscanner);
    if (! b->yy_ch_buf) {
        YY_FATAL_ERROR("out of dynamic memory in Solid_create_buffer()");
    }

    b->yy_is_our_buffer = 1;

    Solid_init_buffer(b, file, yyscanner);

    return b;
}

/** Destroy the buffer.
 * @param b a buffer created with Solid_create_buffer()
 * @param yyscanner The scanner object.
 */
void Solid_delete_buffer(YY_BUFFER_STATE  b, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    if (! b) {
        return;
    }

    if (b == YY_CURRENT_BUFFER) { /* Not sure if we should pop here. */
        YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;
    }

    if (b->yy_is_our_buffer) {
        Solidfree((void *) b->yy_ch_buf, yyscanner);
    }

    Solidfree((void *) b, yyscanner);
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a Solidrestart() or at EOF.
 */
static void Solid_init_buffer(YY_BUFFER_STATE  b, FILE *file, yyscan_t yyscanner)

{
    int oerrno = errno;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    Solid_flush_buffer(b, yyscanner);

    b->yy_input_file = file;
    b->yy_fill_buffer = 1;

    /* If b is the current buffer, then Solid_init_buffer was _probably_
     * called from Solidrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER) {
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

    b->yy_is_interactive = 0;

    errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
void Solid_flush_buffer(YY_BUFFER_STATE  b, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    if (! b) {
        return;
    }

    b->yy_n_chars = 0;

    /* We always need two end-of-buffer characters.  The first causes
     * a transition to the end-of-buffer state.  The second causes
     * a jam in that state.
     */
    b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
    b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

    b->yy_buf_pos = &b->yy_ch_buf[0];

    b->yy_at_bol = 1;
    b->yy_buffer_status = YY_BUFFER_NEW;

    if (b == YY_CURRENT_BUFFER) {
        Solid_load_buffer_state(yyscanner);
    }
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
void Solidpush_buffer_state(YY_BUFFER_STATE new_buffer, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    if (new_buffer == NULL) {
        return;
    }

    Solidensure_buffer_stack(yyscanner);

    /* This block is copied from Solid_switch_to_buffer. */
    if (YY_CURRENT_BUFFER) {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
    }

    /* Only push if top exists. Otherwise, replace top. */
    if (YY_CURRENT_BUFFER) {
        yyg->yy_buffer_stack_top++;
    }
    YY_CURRENT_BUFFER_LVALUE = new_buffer;

    /* copied from Solid_switch_to_buffer. */
    Solid_load_buffer_state(yyscanner);
    yyg->yy_did_buffer_switch_on_eof = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
void Solidpop_buffer_state(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    if (!YY_CURRENT_BUFFER) {
        return;
    }

    Solid_delete_buffer(YY_CURRENT_BUFFER, yyscanner);
    YY_CURRENT_BUFFER_LVALUE = NULL;
    if (yyg->yy_buffer_stack_top > 0) {
        --yyg->yy_buffer_stack_top;
    }

    if (YY_CURRENT_BUFFER) {
        Solid_load_buffer_state(yyscanner);
        yyg->yy_did_buffer_switch_on_eof = 1;
    }
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void Solidensure_buffer_stack(yyscan_t yyscanner)
{
    int num_to_alloc;
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    if (!yyg->yy_buffer_stack) {

        /* First allocation is just for 2 elements, since we don't know if this
         * scanner will even need a stack. We use 2 instead of 1 to avoid an
         * immediate realloc on the next call.
         */
        num_to_alloc = 1;
        yyg->yy_buffer_stack = (struct yy_buffer_state **)Solidalloc
                               (num_to_alloc * sizeof(struct yy_buffer_state *)
                                , yyscanner);
        if (! yyg->yy_buffer_stack) {
            YY_FATAL_ERROR("out of dynamic memory in Solidensure_buffer_stack()");
        }

        memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state *));

        yyg->yy_buffer_stack_max = num_to_alloc;
        yyg->yy_buffer_stack_top = 0;
        return;
    }

    if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1) {

        /* Increase the buffer to prepare for a possible push. */
        int grow_size = 8 /* arbitrary grow size */;

        num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
        yyg->yy_buffer_stack = (struct yy_buffer_state **)Solidrealloc
                               (yyg->yy_buffer_stack,
                                num_to_alloc * sizeof(struct yy_buffer_state *)
                                , yyscanner);
        if (! yyg->yy_buffer_stack) {
            YY_FATAL_ERROR("out of dynamic memory in Solidensure_buffer_stack()");
        }

        /* zero only the new slots.*/
        memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state *));
        yyg->yy_buffer_stack_max = num_to_alloc;
    }
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE Solid_scan_buffer(char *base, yy_size_t  size, yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;

    if (size < 2 ||
            base[size - 2] != YY_END_OF_BUFFER_CHAR ||
            base[size - 1] != YY_END_OF_BUFFER_CHAR)
        /* They forgot to leave room for the EOB's. */
    {
        return 0;
    }

    b = (YY_BUFFER_STATE) Solidalloc(sizeof(struct yy_buffer_state), yyscanner);
    if (! b) {
        YY_FATAL_ERROR("out of dynamic memory in Solid_scan_buffer()");
    }

    b->yy_buf_size = size - 2;  /* "- 2" to take care of EOB's */
    b->yy_buf_pos = b->yy_ch_buf = base;
    b->yy_is_our_buffer = 0;
    b->yy_input_file = 0;
    b->yy_n_chars = b->yy_buf_size;
    b->yy_is_interactive = 0;
    b->yy_at_bol = 1;
    b->yy_fill_buffer = 0;
    b->yy_buffer_status = YY_BUFFER_NEW;

    Solid_switch_to_buffer(b, yyscanner);

    return b;
}

/** Setup the input buffer state to scan a string. The next call to Solidlex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       Solid_scan_bytes() instead.
 */
YY_BUFFER_STATE Solid_scan_string(yyconst char *yystr, yyscan_t yyscanner)
{

    return Solid_scan_bytes(yystr, strlen(yystr), yyscanner);
}

/** Setup the input buffer state to scan the given bytes. The next call to Solidlex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE Solid_scan_bytes(yyconst char *yybytes, int  _yybytes_len, yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;
    char *buf;
    yy_size_t n;
    int i;

    /* Get memory for full buffer, including space for trailing EOB's. */
    n = _yybytes_len + 2;
    buf = (char *) Solidalloc(n, yyscanner);
    if (! buf) {
        YY_FATAL_ERROR("out of dynamic memory in Solid_scan_bytes()");
    }

    for (i = 0; i < _yybytes_len; ++i) {
        buf[i] = yybytes[i];
    }

    buf[_yybytes_len] = buf[_yybytes_len + 1] = YY_END_OF_BUFFER_CHAR;

    b = Solid_scan_buffer(buf, n, yyscanner);
    if (! b) {
        YY_FATAL_ERROR("bad buffer in Solid_scan_bytes()");
    }

    /* It's okay to grow etc. this buffer, and we should throw it
     * away when we're done.
     */
    b->yy_is_our_buffer = 1;

    return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error(yyconst char *msg, yyscan_t yyscanner)
{
    (void) fprintf(stderr, "%s\n", msg);
    exit(YY_EXIT_FAILURE);
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
    do \
    { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        yytext[yyleng] = yyg->yy_hold_char; \
        yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
        yyg->yy_hold_char = *yyg->yy_c_buf_p; \
        *yyg->yy_c_buf_p = '\0'; \
        yyleng = yyless_macro_arg; \
    } \
    while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE Solidget_extra(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yyextra;
}

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int Solidget_lineno(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    if (! YY_CURRENT_BUFFER) {
        return 0;
    }

    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int Solidget_column(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    if (! YY_CURRENT_BUFFER) {
        return 0;
    }

    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *Solidget_in(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *Solidget_out(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int Solidget_leng(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *Solidget_text(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yytext;
}

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void Solidset_extra(YY_EXTRA_TYPE  user_defined, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    yyextra = user_defined;
}

/** Set the current line number.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void Solidset_lineno(int  line_number, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    /* lineno is only valid if an input buffer exists. */
    if (! YY_CURRENT_BUFFER) {
        yy_fatal_error("Solidset_lineno called with no buffer", yyscanner);
    }

    yylineno = line_number;
}

/** Set the current column.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void Solidset_column(int  column_no, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    /* column is only valid if an input buffer exists. */
    if (! YY_CURRENT_BUFFER) {
        yy_fatal_error("Solidset_column called with no buffer", yyscanner);
    }

    yycolumn = column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see Solid_switch_to_buffer
 */
void Solidset_in(FILE   *in_str, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    yyin = in_str;
}

void Solidset_out(FILE   *out_str, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    yyout = out_str;
}

int Solidget_debug(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yy_flex_debug;
}

void Solidset_debug(int  bdebug, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    yy_flex_debug = bdebug;
}

/* Accessor methods for yylval and yylloc */

YYSTYPE *Solidget_lval(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    return yylval;
}

void Solidset_lval(YYSTYPE   *yylval_param, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    yylval = yylval_param;
}

/* User-visible API */

/* Solidlex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int Solidlex_init(yyscan_t *ptr_yy_globals)

{
    if (ptr_yy_globals == NULL) {
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) Solidalloc(sizeof(struct yyguts_t), NULL);

    if (*ptr_yy_globals == NULL) {
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals, 0x00, sizeof(struct yyguts_t));

    return yy_init_globals(*ptr_yy_globals);
}

/* Solidlex_init_extra has the same functionality as Solidlex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to Solidalloc in
 * the yyextra field.
 */

int Solidlex_init_extra(YY_EXTRA_TYPE yy_user_defined, yyscan_t *ptr_yy_globals)

{
    struct yyguts_t dummy_yyguts;

    Solidset_extra(yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL) {
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) Solidalloc(sizeof(struct yyguts_t), &dummy_yyguts);

    if (*ptr_yy_globals == NULL) {
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals, 0x00, sizeof(struct yyguts_t));

    Solidset_extra(yy_user_defined, *ptr_yy_globals);

    return yy_init_globals(*ptr_yy_globals);
}

static int yy_init_globals(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from Solidlex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = 0;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = (char *) 0;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

    /* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * Solidlex_init()
     */
    return 0;
}

/* Solidlex_destroy is for both reentrant and non-reentrant scanners. */
int Solidlex_destroy(yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)yyscanner;

    /* Pop the buffer stack, destroying each element. */
    while (YY_CURRENT_BUFFER) {
        Solid_delete_buffer(YY_CURRENT_BUFFER, yyscanner);
        YY_CURRENT_BUFFER_LVALUE = NULL;
        Solidpop_buffer_state(yyscanner);
    }

    /* Destroy the stack itself. */
    Solidfree(yyg->yy_buffer_stack, yyscanner);
    yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
    Solidfree(yyg->yy_start_stack, yyscanner);
    yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * Solidlex() is called, initialization will occur. */
    yy_init_globals(yyscanner);

    /* Destroy the main struct (reentrant only). */
    Solidfree(yyscanner, yyscanner);
    yyscanner = NULL;
    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy(char *s1, yyconst char *s2, int n, yyscan_t yyscanner)
{
    register int i;
    for (i = 0; i < n; ++i) {
        s1[i] = s2[i];
    }
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen(yyconst char *s, yyscan_t yyscanner)
{
    register int n;
    for (n = 0; s[n]; ++n)
        ;

    return n;
}
#endif

void *Solidalloc(yy_size_t  size, yyscan_t yyscanner)
{
    return (void *) malloc(size);
}

void *Solidrealloc(void *ptr, yy_size_t  size, yyscan_t yyscanner)
{
    /* The cast to (char *) in the following accommodates both
     * implementations that use char* generic pointers, and those
     * that use void* generic pointers.  It works with the latter
     * because both ANSI C and C++ allow castless assignment from
     * any pointer type to void*, and deal with argument conversions
     * as though doing an assignment.
     */
    return (void *) realloc((char *) ptr, size);
}

void Solidfree(void *ptr, yyscan_t yyscanner)
{
    free((char *) ptr);      /* see Solidrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 52 "predicate_lexer.l"

char *PredicateParse_putSymbol(char *_name)
{
    char *p = (char *)malloc(strlen(_name) + 1);
    if (p != NULL) {
        strcpy(p, _name);
    }
    return p;
}

char *PredicateParse_putString(char *_str)
{
    int l = strlen(_str);
    char *p = (char *)malloc(l);
    char *s = _str + 1;
    char *d = p;

    if (p == NULL) {
        return NULL;
    }

    while (s != _str + l - 1) {
        if (*s != '\\') {
            *d++ = *s++;
        } else {
            s++;
            if (s != _str + l - 1) {
                if (*s == '\\') {
                    *d++ = '\\';
                } else if (*s == 'n') {
                    *d++ = '\n';
                } else if (*s == 'r') {
                    *d++ = '\r';
                } else if (*s == 't') {
                    *d++ = '\t';
                }
                s++;
            }
        }
    }
    *d = 0;
    return p;
}

void PredicateParse_initLexer(const char *_code, yyscan_t _scanner)
{
    Solid_switch_to_buffer(Solid_scan_string(_code, _scanner), _scanner);
}

int Solidwrap(yyscan_t _scanner)
{
    struct yyguts_t *yyg = (struct yyguts_t *)_scanner;
    Solid_delete_buffer(YY_CURRENT_BUFFER, _scanner);
    return 1;
}

